@@ -135,6 +135,10 @@ impl Blockstore {
             .db
             .delete_range_cf::<cf::SlotMeta>(&mut write_batch, from_slot, to_slot)
             .is_ok()
+            & self
+                .db
+                .delete_range_cf::<cf::BankHash>(&mut write_batch, from_slot, to_slot)
+                .is_ok()
             & self
                 .db
                 .delete_range_cf::<cf::Root>(&mut write_batch, from_slot, to_slot)
@@ -264,6 +268,10 @@ impl Blockstore {
                 .orphans_cf
                 .compact_range(from_slot, to_slot)
                 .unwrap_or(false)
+            && self
+                .bank_hash_cf
+                .compact_range(from_slot, to_slot)
+                .unwrap_or(false)
             && self
                 .index_cf
                 .compact_range(from_slot, to_slot)
